@@ -30,6 +30,8 @@ class CMaintenance extends CApiService {
 	protected $tableAlias = 'm';
 	protected $sortColumns = ['maintenanceid', 'name', 'maintenance_type', 'active_till', 'active_since'];
 
+	private const MAX_TIMEPERIOD = 999 * SEC_PER_DAY + 23 * SEC_PER_HOUR + 59 * SEC_PER_MIN;
+
 	/**
 	 * Get maintenances data.
 	 *
@@ -95,8 +97,8 @@ class CMaintenance extends CApiService {
 			}
 
 			$permission_condition = $options['editable']
-				? ' AND (p.permission IS NULL OR p.permission < '.PERM_READ_WRITE.')'
-				: ' AND p.permission IS NULL';
+				? ' AND (p.hgsetid IS NULL OR p.permission < '.PERM_READ_WRITE.')'
+				: ' AND p.hgsetid IS NULL';
 
 			$sqlParts['where'][] = 'NOT EXISTS ('.
 				'SELECT NULL'.
@@ -200,11 +202,12 @@ class CMaintenance extends CApiService {
 
 		if ($result) {
 			$result = $this->addRelatedObjects($options, $result);
+
+			if (!$options['preservekeys']) {
+				$result = array_values($result);
+			}
 		}
 
-		if (!$options['preservekeys']) {
-			$result = zbx_cleanHashes($result);
-		}
 		return $result;
 	}
 
@@ -268,7 +271,7 @@ class CMaintenance extends CApiService {
 				'hostid' =>				['type' => API_ID, 'flags' => API_REQUIRED]
 			]],
 			'timeperiods' =>		['type' => API_OBJECTS, 'flags' => API_REQUIRED | API_NOT_EMPTY | API_NORMALIZE, 'fields' => [
-				'period' =>				['type' => API_TIME_UNIT, 'in' => implode(':', [5 * SEC_PER_MIN, ZBX_MAX_INT32]), 'default' => SEC_PER_HOUR],
+				'period' =>				['type' => API_TIME_UNIT, 'in' => implode(':', [5 * SEC_PER_MIN, self::MAX_TIMEPERIOD]), 'default' => SEC_PER_HOUR],
 				'timeperiod_type' =>	['type' => API_INT32, 'in' => implode(',', [TIMEPERIOD_TYPE_ONETIME, TIMEPERIOD_TYPE_DAILY, TIMEPERIOD_TYPE_WEEKLY, TIMEPERIOD_TYPE_MONTHLY]), 'default' => DB::getDefault('timeperiods', 'timeperiod_type')],
 				'start_date' =>			['type' => API_MULTIPLE, 'rules' => [
 											['if' => ['field' => 'timeperiod_type', 'in' => implode(',', [TIMEPERIOD_TYPE_ONETIME])], 'type' => API_TIMESTAMP, 'default' => time()],
@@ -279,7 +282,8 @@ class CMaintenance extends CApiService {
 											['else' => true, 'type' => API_UNEXPECTED]
 				]],
 				'every' =>				['type' => API_MULTIPLE, 'rules' => [
-											['if' => ['field' => 'timeperiod_type', 'in' => implode(',', [TIMEPERIOD_TYPE_DAILY, TIMEPERIOD_TYPE_WEEKLY])], 'type' => API_INT32, 'in' => implode(':', [1, ZBX_MAX_INT32]), 'default' => DB::getDefault('timeperiods', 'every')],
+											['if' => ['field' => 'timeperiod_type', 'in' => implode(',', [TIMEPERIOD_TYPE_DAILY])], 'type' => API_INT32, 'in' => implode(':', [1, 999]), 'default' => DB::getDefault('timeperiods', 'every')],
+											['if' => ['field' => 'timeperiod_type', 'in' => implode(',', [TIMEPERIOD_TYPE_WEEKLY])], 'type' => API_INT32, 'in' => implode(':', [1, 99]), 'default' => DB::getDefault('timeperiods', 'every')],
 											['if' => ['field' => 'timeperiod_type', 'in' => implode(',', [TIMEPERIOD_TYPE_MONTHLY])], 'type' => API_INT32, 'in' => implode(',', [MONTH_WEEK_FIRST, MONTH_WEEK_SECOND, MONTH_WEEK_THIRD, MONTH_WEEK_FOURTH, MONTH_WEEK_LAST]), 'default' => DB::getDefault('timeperiods', 'every')],
 											['else' => true, 'type' => API_UNEXPECTED]
 				]],
@@ -368,7 +372,7 @@ class CMaintenance extends CApiService {
 	 *
 	 * @throws APIException if the input is invalid.
 	 */
-	protected function validateUpdate(array &$maintenances, array &$db_maintenances = null): void {
+	protected function validateUpdate(array &$maintenances, ?array &$db_maintenances = null): void {
 		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE | API_ALLOW_UNEXPECTED, 'uniq' => [['maintenanceid']], 'fields' => [
 			'maintenanceid' =>	['type' => API_ID, 'flags' => API_REQUIRED]
 		]];
@@ -420,7 +424,7 @@ class CMaintenance extends CApiService {
 				'hostid' =>				['type' => API_ID, 'flags' => API_REQUIRED]
 			]],
 			'timeperiods' =>		['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'fields' => [
-				'period' =>				['type' => API_TIME_UNIT, 'in' => implode(':', [5 * SEC_PER_MIN, ZBX_MAX_INT32]), 'default' => SEC_PER_HOUR],
+				'period' =>				['type' => API_TIME_UNIT, 'in' => implode(':', [5 * SEC_PER_MIN, self::MAX_TIMEPERIOD]), 'default' => SEC_PER_HOUR],
 				'timeperiod_type' =>	['type' => API_INT32, 'in' => implode(',', [TIMEPERIOD_TYPE_ONETIME, TIMEPERIOD_TYPE_DAILY, TIMEPERIOD_TYPE_WEEKLY, TIMEPERIOD_TYPE_MONTHLY]), 'default' => DB::getDefault('timeperiods', 'timeperiod_type')],
 				'start_date' =>			['type' => API_MULTIPLE, 'rules' => [
 											['if' => ['field' => 'timeperiod_type', 'in' => implode(',', [TIMEPERIOD_TYPE_ONETIME])], 'type' => API_TIMESTAMP, 'default' => time()],
@@ -431,7 +435,8 @@ class CMaintenance extends CApiService {
 											['else' => true, 'type' => API_UNEXPECTED]
 				]],
 				'every' =>				['type' => API_MULTIPLE, 'rules' => [
-											['if' => ['field' => 'timeperiod_type', 'in' => implode(',', [TIMEPERIOD_TYPE_DAILY, TIMEPERIOD_TYPE_WEEKLY])], 'type' => API_INT32, 'in' => implode(':', [1, ZBX_MAX_INT32]), 'default' => DB::getDefault('timeperiods', 'every')],
+											['if' => ['field' => 'timeperiod_type', 'in' => implode(',', [TIMEPERIOD_TYPE_DAILY])], 'type' => API_INT32, 'in' => implode(':', [1, 999]), 'default' => DB::getDefault('timeperiods', 'every')],
+											['if' => ['field' => 'timeperiod_type', 'in' => implode(',', [TIMEPERIOD_TYPE_WEEKLY])], 'type' => API_INT32, 'in' => implode(':', [1, 99]), 'default' => DB::getDefault('timeperiods', 'every')],
 											['if' => ['field' => 'timeperiod_type', 'in' => implode(',', [TIMEPERIOD_TYPE_MONTHLY])], 'type' => API_INT32, 'in' => implode(',', [MONTH_WEEK_FIRST, MONTH_WEEK_SECOND, MONTH_WEEK_THIRD, MONTH_WEEK_FOURTH, MONTH_WEEK_LAST]), 'default' => DB::getDefault('timeperiods', 'every')],
 											['else' => true, 'type' => API_UNEXPECTED]
 				]],
@@ -520,11 +525,7 @@ class CMaintenance extends CApiService {
 			'where' => ['maintenanceid' => $maintenanceids]
 		]);
 
-		DB::delete('maintenances_windows', ['maintenanceid' => $maintenanceids]);
 		DB::delete('timeperiods', ['timeperiodid' => array_column($maintenances_windows, 'timeperiodid')]);
-		DB::delete('maintenances_hosts', ['maintenanceid' => $maintenanceids]);
-		DB::delete('maintenances_groups', ['maintenanceid' => $maintenanceids]);
-		DB::delete('maintenance_tag', ['maintenanceid' => $maintenanceids]);
 		DB::delete('maintenances', ['maintenanceid' => $maintenanceids]);
 
 		self::addAuditLog(CAudit::ACTION_DELETE, CAudit::RESOURCE_MAINTENANCE, $db_maintenances);
@@ -538,7 +539,7 @@ class CMaintenance extends CApiService {
 	 *
 	 * @throws APIException if the input is invalid.
 	 */
-	private function validateDelete(array $maintenanceids, array &$db_maintenances = null): void {
+	private function validateDelete(array $maintenanceids, ?array &$db_maintenances = null): void {
 		$api_input_rules = ['type' => API_IDS, 'flags' => API_NOT_EMPTY, 'uniq' => true];
 
 		if (!CApiInputValidator::validate($api_input_rules, $maintenanceids, '/', $error)) {
@@ -613,7 +614,7 @@ class CMaintenance extends CApiService {
 	 *
 	 * @throws APIException if maintenance names are not unique.
 	 */
-	protected static function checkDuplicates(array $maintenances, array $db_maintenances = null): void {
+	protected static function checkDuplicates(array $maintenances, ?array $db_maintenances = null): void {
 		$names = [];
 
 		foreach ($maintenances as $maintenance) {
@@ -650,7 +651,7 @@ class CMaintenance extends CApiService {
 	 *
 	 * @throws APIException if groups are not valid.
 	 */
-	private static function checkGroups(array $maintenances, array $db_maintenances = null): void {
+	private static function checkGroups(array $maintenances, ?array $db_maintenances = null): void {
 		$edit_groupids = [];
 
 		foreach ($maintenances as $maintenance) {
@@ -696,7 +697,7 @@ class CMaintenance extends CApiService {
 	 *
 	 * @throws APIException if hosts are not valid.
 	 */
-	private static function checkHosts(array $maintenances, array $db_maintenances = null): void {
+	private static function checkHosts(array $maintenances, ?array $db_maintenances = null): void {
 		$edit_hostids = [];
 
 		foreach ($maintenances as $maintenance) {
@@ -741,7 +742,7 @@ class CMaintenance extends CApiService {
 	 * @param array      $maintenances
 	 * @param array|null $db_maintenances
 	 */
-	private static function updateTags(array &$maintenances, array $db_maintenances = null): void {
+	private static function updateTags(array &$maintenances, ?array $db_maintenances = null): void {
 		$ins_maintenance_tags = [];
 		$del_maintenancetagids = [];
 
@@ -809,7 +810,7 @@ class CMaintenance extends CApiService {
 	 * @param array      $maintenances
 	 * @param array|null $db_maintenances
 	 */
-	private static function updateGroups(array &$maintenances, array $db_maintenances = null): void {
+	private static function updateGroups(array &$maintenances, ?array $db_maintenances = null): void {
 		$ins_groups = [];
 		$del_groupids = [];
 
@@ -871,7 +872,7 @@ class CMaintenance extends CApiService {
 	 * @param array      $maintenances
 	 * @param array|null $db_maintenances
 	 */
-	private static function updateHosts(array &$maintenances, array $db_maintenances = null): void {
+	private static function updateHosts(array &$maintenances, ?array $db_maintenances = null): void {
 		$ins_maintenances_hosts = [];
 		$del_maintenance_hostids = [];
 
@@ -935,7 +936,7 @@ class CMaintenance extends CApiService {
 	 * @param array      $maintenances
 	 * @param array|null $db_maintenances
 	 */
-	private static function updateTimeperiods(array &$maintenances, array $db_maintenances = null): void {
+	private static function updateTimeperiods(array &$maintenances, ?array $db_maintenances = null): void {
 		$ins_timeperiods = [];
 		$ins_maintenances_windows = [];
 		$del_timeperiodids = [];
