@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -34,10 +29,26 @@ class CUserMacro extends CApiService {
 		'deleteglobal' => ['min_user_type' => USER_TYPE_SUPER_ADMIN]
 	];
 
+	public const OUTPUT_FIELDS = ['hostmacroid', 'hostid', 'macro', 'value', 'type', 'description', 'automatic',
+		'config'
+	];
+
 	protected $tableName = 'hostmacro';
 	protected $tableAlias = 'hm';
 	protected $sortColumns = ['macro'];
 
+	public static function getOutputFieldsOnHost(): array {
+		return array_diff(self::OUTPUT_FIELDS, ['hostid', 'config']);
+	}
+
+	public static function getOutputFieldsOnHostPrototype(): array {
+		return array_diff(self::OUTPUT_FIELDS, ['hostid', 'automatic', 'config']);
+	}
+
+	public static function getOutputFieldsOnTemplate(): array {
+		return array_diff(self::OUTPUT_FIELDS, ['hostid', 'automatic']);
+	}
+
 	/**
 	 * Get UserMacros data.
 	 *
@@ -48,7 +59,6 @@ class CUserMacro extends CApiService {
 	 * @param array $options['globalmacroids'] global macros ids
 	 * @param array $options['templateids'] template ids
 	 * @param boolean $options['globalmacro'] only global macros
-	 * @param boolean $options['selectGroups'] select groups - deprecated!
 	 * @param boolean $options['selectHostGroups'] select host groups
 	 * @param boolean $options['selectTemplateGroups'] select template groups
 	 * @param boolean $options['selectHosts'] select hosts
@@ -58,7 +68,6 @@ class CUserMacro extends CApiService {
 	 */
 	public function get($options = []) {
 		$result = [];
-		$userid = self::$userData['userid'];
 
 		$sqlParts = [
 			'select'	=> ['macros' => 'hm.hostmacroid'],
@@ -95,7 +104,6 @@ class CUserMacro extends CApiService {
 			'searchWildcardsEnabled'	=> null,
 			// output
 			'output'					=> API_OUTPUT_EXTEND,
-			'selectGroups'				=> null,
 			'selectHostGroups'			=> null,
 			'selectTemplateGroups'		=> null,
 			'selectHosts'				=> null,
@@ -108,29 +116,20 @@ class CUserMacro extends CApiService {
 		];
 		$options = zbx_array_merge($defOptions, $options);
 
-		$this->checkDeprecatedParam($options, 'selectGroups');
-
 		// editable + PERMISSION CHECK
 		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
-			if ($options['editable'] && !is_null($options['globalmacro'])) {
-				return [];
+			if (($options['editable'] && !is_null($options['globalmacro'])) || self::$userData['ugsetid'] == 0) {
+				return $options['countOutput'] ? '0' : [];
 			}
 			else {
-				$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ;
+				$sqlParts['from'][] = 'host_hgset hh';
+				$sqlParts['from'][] = 'permission p';
+				$sqlParts['where'][] = 'hm.hostid=hh.hostid';
+				$sqlParts['where'][] = 'hh.hgsetid=p.hgsetid';
 
-				$userGroups = getUserGroupsByUserId($userid);
-
-				$sqlParts['where'][] = 'EXISTS ('.
-						'SELECT NULL'.
-						' FROM hosts_groups hgg'.
-							' JOIN rights r'.
-								' ON r.id=hgg.groupid'.
-									' AND '.dbConditionInt('r.groupid', $userGroups).
-						' WHERE hm.hostid=hgg.hostid'.
-						' GROUP BY hgg.hostid'.
-						' HAVING MIN(r.permission)>'.PERM_DENY.
-							' AND MAX(r.permission)>='.zbx_dbstr($permission).
-						')';
+				if ($options['editable']) {
+					$sqlParts['where'][] = 'p.permission='.PERM_READ_WRITE;
+				}
 			}
 		}
 
@@ -141,7 +140,6 @@ class CUserMacro extends CApiService {
 			$options['triggerids'] = null;
 			$options['hostids'] = null;
 			$options['itemids'] = null;
-			$options['selectGroups'] = null;
 			$options['selectHostGroups'] = null;
 			$options['selectTemplateGroups'] = null;
 			$options['selectTemplates'] = null;
@@ -219,6 +217,7 @@ class CUserMacro extends CApiService {
 		}
 		// init HOSTS
 		else {
+			$options['output'] = $this->outputExtend($options['output'], ['hostid']);
 			$sqlParts = $this->applyQueryFilterOptions('hostmacro', 'hm', $options, $sqlParts);
 			$sqlParts = $this->applyQueryOutputOptions('hostmacro', 'hm', $options, $sqlParts);
 			$res = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
@@ -239,11 +238,10 @@ class CUserMacro extends CApiService {
 		if ($result) {
 			$result = $this->addRelatedObjects($options, $result);
 			$result = $this->unsetExtraFields($result, ['hostid', 'type'], $options['output']);
-		}
 
-		// removing keys (hash -> array)
-		if (!$options['preservekeys']) {
-			$result = zbx_cleanHashes($result);
+			if (!$options['preservekeys']) {
+				$result = array_values($result);
+			}
 		}
 
 		return $result;
@@ -330,7 +328,7 @@ class CUserMacro extends CApiService {
 	 *
 	 * @throws APIException if the input is invalid
 	 */
-	private function validateUpdateGlobal(array &$globalmacros, array &$db_globalmacros = null) {
+	private function validateUpdateGlobal(array &$globalmacros, ?array &$db_globalmacros = null) {
 		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['globalmacroid'], ['macro']], 'fields' => [
 			'globalmacroid' =>	['type' => API_ID, 'flags' => API_REQUIRED],
 			'macro' =>			['type' => API_USER_MACRO, 'length' => DB::getFieldLength('globalmacro', 'macro')],
@@ -392,7 +390,7 @@ class CUserMacro extends CApiService {
 	 *
 	 * @throws APIException if macros already exists.
 	 */
-	private function checkDuplicates(array $globalmacros, array $db_globalmacros = null): void {
+	private function checkDuplicates(array $globalmacros, ?array $db_globalmacros = null): void {
 		$macros = [];
 
 		foreach ($globalmacros as $globalmacro) {
@@ -443,7 +441,7 @@ class CUserMacro extends CApiService {
 	 *
 	 * @throws APIException if the input is invalid.
 	 */
-	private function validateDeleteGlobal(array &$globalmacroids, array &$db_globalmacros = null) {
+	private function validateDeleteGlobal(array &$globalmacroids, ?array &$db_globalmacros = null) {
 		$api_input_rules = ['type' => API_IDS, 'flags' => API_NOT_EMPTY, 'uniq' => true];
 
 		if (!CApiInputValidator::validate($api_input_rules, $globalmacroids, '/', $error)) {
@@ -475,13 +473,15 @@ class CUserMacro extends CApiService {
 									['if' => ['field' => 'type', 'in' => implode(',', [ZBX_MACRO_TYPE_TEXT, ZBX_MACRO_TYPE_SECRET])], 'type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'value')],
 									['if' => ['field' => 'type', 'in' => implode(',', [ZBX_MACRO_TYPE_VAULT])], 'type' => API_VAULT_SECRET, 'provider' => CSettingsHelper::get(CSettingsHelper::VAULT_PROVIDER), 'length' => DB::getFieldLength('hostmacro', 'value')]
 			]],
-			'description' =>	['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'description')]
+			'description' =>	['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'description')],
+			'config' => 		['type' => API_ANY]
 		]];
 
 		if (!CApiInputValidator::validate($api_input_rules, $hostmacros, '/', $error)) {
 			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 		}
 
+		self::validateMacroConfig($hostmacros);
 		$this->checkHostPermissions(array_unique(array_column($hostmacros, 'hostid')));
 		$this->checkHostDuplicates($hostmacros);
 	}
@@ -515,6 +515,8 @@ class CUserMacro extends CApiService {
 			$hostmacro['hostmacroid'] = $hostmacroids[$index];
 		}
 		unset($hostmacro);
+
+		$this->updateRealHostmacroConfig($hostmacros);
 	}
 
 	/**
@@ -522,14 +524,15 @@ class CUserMacro extends CApiService {
 	 *
 	 * @throws APIException if the input is invalid.
 	 */
-	protected function validateUpdate(array &$hostmacros, array &$db_hostmacros = null) {
+	protected function validateUpdate(array &$hostmacros, ?array &$db_hostmacros = null) {
 		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['hostmacroid']], 'fields' => [
 			'hostmacroid' =>	['type' => API_ID, 'flags' => API_REQUIRED],
 			'macro' =>			['type' => API_USER_MACRO, 'length' => DB::getFieldLength('hostmacro', 'macro')],
 			'type' =>			['type' => API_INT32, 'in' => implode(',', [ZBX_MACRO_TYPE_TEXT, ZBX_MACRO_TYPE_SECRET, ZBX_MACRO_TYPE_VAULT])],
 			'value' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'value')],
 			'description' =>	['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'description')],
-			'automatic' =>		['type' => API_INT32, 'in' => implode(',', [ZBX_USERMACRO_MANUAL])]
+			'automatic' =>		['type' => API_INT32, 'in' => implode(',', [ZBX_USERMACRO_MANUAL])],
+			'config' => 		['type' => API_ANY]
 		]];
 
 		if (!CApiInputValidator::validate($api_input_rules, $hostmacros, '/', $error)) {
@@ -537,7 +540,7 @@ class CUserMacro extends CApiService {
 		}
 
 		$db_hostmacros = $this->get([
-			'output' => ['hostmacroid', 'hostid', 'macro', 'type', 'description', 'automatic'],
+			'output' => ['hostmacroid', 'hostid', 'macro', 'type', 'description', 'automatic', 'config'],
 			'hostmacroids' => array_column($hostmacros, 'hostmacroid'),
 			'editable' => true,
 			'inherited' => false,
@@ -600,6 +603,7 @@ class CUserMacro extends CApiService {
 			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 		}
 
+		self::validateMacroConfig($hostmacros, $db_hostmacros);
 		$this->checkHostDuplicates($hostmacros, $db_hostmacros);
 	}
 
@@ -616,7 +620,7 @@ class CUserMacro extends CApiService {
 	 *
 	 * @throws APIException if any of the given macros already exist.
 	 */
-	private function checkHostDuplicates(array $hostmacros, array $db_hostmacros = null) {
+	private function checkHostDuplicates(array $hostmacros, ?array $db_hostmacros = null) {
 		$macro_names = [];
 		$existing_macros = [];
 
@@ -677,6 +681,74 @@ class CUserMacro extends CApiService {
 		}
 	}
 
+	private static function validateMacroConfig(array $hostmacros, ?array $db_hostmacros = null): void {
+		$api_input_rules = CTemplate::getMacroConfigValidationRules();
+
+		$all_hostids = array_flip(array_column($hostmacros, 'hostid'));
+
+		if ($db_hostmacros !== null) {
+			$all_hostids += array_flip(array_column($db_hostmacros, 'hostid'));
+		}
+
+		$templates = API::Template()->get([
+			'output' => [],
+			'templateids' => array_keys($all_hostids),
+			'preservekeys' => true
+		]);
+
+		foreach ($hostmacros as $index => $hostmacro) {
+			// Config is supported only for template macros.
+			if (!array_key_exists('config', $hostmacro)) {
+				continue;
+			}
+
+			$path = '/'.($index + 1);
+
+			if (!array_key_exists($hostmacro['hostid'], $templates)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid parameter "%1$s": %2$s.', $path,
+					_s('unexpected parameter "%1$s"', 'config'))
+				);
+			}
+
+			if (!$hostmacro['config']) {
+				continue;
+			}
+
+			$path .= '/config';
+
+			if ($db_hostmacros !== null && array_key_exists($hostmacro['hostmacroid'], $db_hostmacros)) {
+				$db_hostmacro = $db_hostmacros[$hostmacro['hostmacroid']];
+				$hostmacro['config'] += ['type' => $db_hostmacro['config']['type']];
+				self::addRequiredFieldsByMacroConfigType($hostmacro['config'], $db_hostmacro['config']);
+			}
+
+			if (!CApiInputValidator::validate($api_input_rules, $hostmacro['config'], $path, $error)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+			}
+
+			if (array_key_exists('options', $hostmacro['config'])) {
+				$path_options = $path . '/options';
+				$encoded = json_encode($hostmacro['config']['options'], JSON_THROW_ON_ERROR);
+
+				if (mb_strlen($encoded) > DB::getFieldLength('hostmacro_config', 'options')) {
+					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid parameter "%1$s": %2$s.', $path_options,
+						_('value is too long')
+					));
+				}
+			}
+		}
+	}
+
+	private static function addRequiredFieldsByMacroConfigType(array &$config, array $db_config): void {
+		if ($config['type'] == ZBX_WIZARD_FIELD_TEXT) {
+			$config += array_intersect_key($db_config, array_flip(['label']));
+		}
+
+		if (in_array($config['type'], [ZBX_WIZARD_FIELD_LIST, ZBX_WIZARD_FIELD_CHECKBOX])) {
+			$config += array_intersect_key($db_config, array_flip(['label', 'options']));
+		}
+	}
+
 	/**
 	 * Update host macros.
 	 *
@@ -721,6 +793,135 @@ class CUserMacro extends CApiService {
 		if ($upd_hostmacros) {
 			DB::update('hostmacro', $upd_hostmacros);
 		}
+
+		self::updateRealHostmacroConfig($hostmacros, $db_hostmacros);
+	}
+
+	/**
+	 * Updates hostmacros_config records in the database.
+	 *
+	 * @param array      $hostmacros
+	 * @param array|null $db_hostmacros
+	 */
+	private static function updateRealHostmacroConfig(array $hostmacros, ?array $db_hostmacros = null): void {
+		$ins_hostmacros_configs = [];
+		$upd_hostmacros_configs = [];
+		$del_hostmacros_configids = [];
+
+		self::addFieldDefaultsByMacroConfigType($hostmacros, $db_hostmacros);
+		self::prepareMacroConfigOptionsForDb($hostmacros, $db_hostmacros);
+
+		foreach ($hostmacros as $hostmacro) {
+			if (!array_key_exists('config', $hostmacro) || !$hostmacro['config']) {
+				continue;
+			}
+
+			if ($db_hostmacros !== null) {
+				$db_hostmacro = $db_hostmacros[$hostmacro['hostmacroid']];
+				$db_hostmacro_config = array_key_exists('config', $db_hostmacro) ? $db_hostmacro['config'] : [];
+
+				if ($db_hostmacro_config
+						&& $db_hostmacro_config['type'] != ZBX_WIZARD_FIELD_NOCONF
+						&& array_key_exists('type', $hostmacro['config'])
+						&& $hostmacro['config']['type'] == ZBX_WIZARD_FIELD_NOCONF) {
+					$del_hostmacros_configids[] = $hostmacro['hostmacroid'];
+					continue;
+				}
+
+				if ($db_hostmacro_config
+						&& $db_hostmacro_config['type'] == ZBX_WIZARD_FIELD_NOCONF
+						&& array_key_exists('type', $hostmacro['config'])
+						&& $hostmacro['config']['type'] != ZBX_WIZARD_FIELD_NOCONF) {
+					$ins_hostmacros_configs[] = ['hostmacroid' => $hostmacro['hostmacroid']] + $hostmacro['config'];
+					continue;
+				}
+
+				$upd_hostmacro_config = DB::getUpdatedValues('hostmacro_config', $hostmacro['config'],
+					$db_hostmacro_config
+				);
+
+				if ($upd_hostmacro_config) {
+					$upd_hostmacros_configs[] = [
+						'values' => $upd_hostmacro_config,
+						'where' => ['hostmacroid' => $hostmacro['hostmacroid']]
+					];
+				}
+			}
+			else {
+				if ($hostmacro['config']['type'] != ZBX_WIZARD_FIELD_NOCONF) {
+					$ins_hostmacros_configs[] = ['hostmacroid' => $hostmacro['hostmacroid']] + $hostmacro['config'];
+				}
+			}
+		}
+
+		if ($ins_hostmacros_configs) {
+			DB::insert('hostmacro_config', $ins_hostmacros_configs, false);
+		}
+
+		if ($upd_hostmacros_configs) {
+			DB::update('hostmacro_config', $upd_hostmacros_configs);
+		}
+
+		if ($del_hostmacros_configids) {
+			DB::delete('hostmacro_config', ['hostmacroid' => $del_hostmacros_configids]);
+		}
+	}
+
+	private static function addFieldDefaultsByMacroConfigType(array &$hostmacros, ?array &$db_hostmacros = null): void {
+		if ($db_hostmacros === null) {
+			return;
+		}
+
+		$field_defaults = DB::getDefaults('hostmacro_config');
+		$field_defaults['options'] = [];
+
+		foreach ($hostmacros as &$macro) {
+			if (array_key_exists($macro['hostmacroid'], $db_hostmacros)
+				&& array_key_exists('config', $macro)
+				&& array_key_exists('type', $macro['config'])
+				&& $macro['config']['type'] != $db_hostmacros[$macro['hostmacroid']]['config']['type']
+			) {
+				$field_names = [];
+
+				if ($macro['config']['type'] == ZBX_WIZARD_FIELD_NOCONF) {
+					$field_names = ['priority', 'section_name', 'label', 'description', 'required', 'regex', 'options'];
+				}
+				elseif ($macro['config']['type'] == ZBX_WIZARD_FIELD_TEXT) {
+					$field_names = ['options'];
+				}
+				elseif ($macro['config']['type'] == ZBX_WIZARD_FIELD_LIST) {
+					$field_names = ['regex'];
+				}
+				elseif ($macro['config']['type'] == ZBX_WIZARD_FIELD_CHECKBOX) {
+					$field_names = ['required', 'regex'];
+				}
+
+				$macro['config'] += array_intersect_key($field_defaults, array_flip($field_names));
+			}
+		}
+		unset($macro);
+	}
+
+	private static function prepareMacroConfigOptionsForDb(array &$hostmacros, ?array &$db_hostmacros = null): void {
+		foreach ($hostmacros as &$macro) {
+			if (array_key_exists('config', $macro) && array_key_exists('options', $macro['config'])) {
+				$macro['config']['options'] = $macro['config']['options']
+					? json_encode($macro['config']['options'])
+					: '';
+			}
+		}
+		unset($macro);
+
+		if ($db_hostmacros !== null) {
+			foreach ($db_hostmacros as &$db_macro) {
+				if (array_key_exists('config', $db_macro) && array_key_exists('options', $db_macro['config'])) {
+					$db_macro['config']['options'] = $db_macro['config']['options']
+						? json_encode($db_macro['config']['options'])
+						: '';
+				}
+			}
+			unset($db_macro);
+		}
 	}
 
 	/**
@@ -729,7 +930,7 @@ class CUserMacro extends CApiService {
 	 *
 	 * @throws APIException if the input is invalid.
 	 */
-	protected function validateDelete(array &$hostmacroids, array &$db_hostmacros = null) {
+	protected function validateDelete(array &$hostmacroids, ?array &$db_hostmacros = null) {
 		$api_input_rules = ['type' => API_IDS, 'flags' => API_NOT_EMPTY, 'uniq' => true];
 
 		if (!CApiInputValidator::validate($api_input_rules, $hostmacroids, '/', $error)) {
@@ -827,11 +1028,11 @@ class CUserMacro extends CApiService {
 	 *
 	 * @return array
 	 */
-	private function getMacrosToInherit(array $hostmacros, array $db_hostmacros = null): array {
+	private function getMacrosToInherit(array $hostmacros, ?array $db_hostmacros = null): array {
 		$templated_host_prototypeids = DBfetchColumn(DBselect(
 			'SELECT hd.hostid'.
 			' FROM host_discovery hd,items i,hosts h'.
-			' WHERE hd.parent_itemid=i.itemid'.
+			' WHERE hd.lldruleid=i.itemid'.
 				' AND i.hostid=h.hostid'.
 				' AND h.status='.HOST_STATUS_TEMPLATE.
 				' AND '.dbConditionId('hd.hostid', array_unique(array_column($hostmacros, 'hostid')))
@@ -877,8 +1078,8 @@ class CUserMacro extends CApiService {
 	 * @param array  $upd_hostmacros
 	 * @param array  $db_hostmacros
 	 */
-	private function prepareInheritedObjects(array $tpl_hostmacros, array &$ins_hostmacros = null,
-			array &$upd_hostmacros = null, array &$db_hostmacros = null): void {
+	private function prepareInheritedObjects(array $tpl_hostmacros, ?array &$ins_hostmacros = null,
+			?array &$upd_hostmacros = null, ?array &$db_hostmacros = null): void {
 		$ins_hostmacros = [];
 		$upd_hostmacros = [];
 		$db_hostmacros = [];
@@ -998,9 +1199,8 @@ class CUserMacro extends CApiService {
 		$sqlParts = parent::applyQueryOutputOptions($tableName, $tableAlias, $options, $sqlParts);
 
 		if ($options['output'] != API_OUTPUT_COUNT && $options['globalmacro'] === null) {
-			if ($options['selectGroups'] !== null || $options['selectHostGroups'] !== null
-					|| $options['selectTemplateGroups'] !== null || $options['selectHosts'] !== null
-					|| $options['selectTemplates'] !== null) {
+			if ($options['selectHostGroups'] !== null || $options['selectTemplateGroups'] !== null
+					|| $options['selectHosts'] !== null || $options['selectTemplates'] !== null) {
 				$sqlParts = $this->addQuerySelect($this->fieldId('hostid'), $sqlParts);
 			}
 		}
@@ -1053,93 +1253,142 @@ class CUserMacro extends CApiService {
 		$result = parent::addRelatedObjects($options, $result);
 
 		if ($options['globalmacro'] === null) {
-			/*
-			 * Adding objects
-			 */
-			// adding groups
-			$this->addRelatedGroups($options, $result, 'selectGroups');
-			$this->addRelatedGroups($options, $result, 'selectHostGroups');
-			$this->addRelatedGroups($options, $result, 'selectTemplateGroups');
-
-			// adding templates
-			if ($options['selectTemplates'] !== null && $options['selectTemplates'] != API_OUTPUT_COUNT) {
-				$relationMap = $this->createRelationMap($result, 'hostmacroid', 'hostid');
-				$templates = API::Template()->get([
-					'output' => $options['selectTemplates'],
-					'templateids' => $relationMap->getRelatedIds(),
-					'preservekeys' => true
-				]);
-				$result = $relationMap->mapMany($result, $templates, 'templates');
-			}
-
-			// adding templates
-			if ($options['selectHosts'] !== null && $options['selectHosts'] != API_OUTPUT_COUNT) {
-				$relationMap = $this->createRelationMap($result, 'hostmacroid', 'hostid');
-				$templates = API::Host()->get([
-					'output' => $options['selectHosts'],
-					'hostids' => $relationMap->getRelatedIds(),
-					'preservekeys' => true
-				]);
-				$result = $relationMap->mapMany($result, $templates, 'hosts');
-			}
+			$this->addRelatedConfig($options, $result);
+			$this->addRelatedHostGroups($options, $result);
+			$this->addRelatedTemplateGroups($options, $result);
+			$this->addRelatedTemplates($options, $result);
+			$this->addRelatedHosts($options, $result);
 		}
 
 		return $result;
 	}
 
-	/**
-	 * Adds related host or template groups requested by "select*" options to the resulting object set.
-	 *
-	 * @param array  $options [IN] Original input options.
-	 * @param array  $result  [IN/OUT] Result output.
-	 * @param string $option  [IN] Possible values:
-	 *                               - "selectGroups" (deprecated);
-	 *                               - "selectHostGroups";
-	 *                               - "selectTemplateGroups".
-	 */
-	private function addRelatedGroups(array $options, array &$result, string $option): void {
-		if ($options[$option] === null || $options[$option] === API_OUTPUT_COUNT) {
+	private function addRelatedConfig(array $options, array &$result): void {
+		if (!$this->outputIsRequested('config', $options['output'])) {
 			return;
 		}
 
-		$res = DBselect(
+		$templates = API::Template()->get([
+			'output' => [],
+			'templateids' => array_keys(array_flip(array_column($result, 'hostid'))),
+			'preservekeys' => true
+		]);
+
+		if (!$templates) {
+			return;
+		}
+
+		$resource = DB::select('hostmacro_config', [
+			'output' => ['type', 'priority', 'section_name', 'label', 'description', 'required', 'regex', 'options'],
+			'hostmacroids' => array_keys($result),
+			'preservekeys' => true
+		]);
+
+		foreach ($result as $hostmacroid => $hostmacro) {
+			if (!array_key_exists($hostmacro['hostid'], $templates)) {
+				continue;
+			}
+
+			$result[$hostmacroid]['config'] = array_key_exists($hostmacroid, $resource)
+				? $resource[$hostmacroid]
+				: DB::getDefaults('hostmacro_config');
+
+			$result[$hostmacroid]['config']['options'] = $result[$hostmacroid]['config']['options'] !== ''
+				? json_decode($result[$hostmacroid]['config']['options'], true)
+				: [];
+		}
+	}
+
+	private function addRelatedHostGroups(array $options, array &$result): void {
+		if ($options['selectHostGroups'] === null || $options['selectHostGroups'] === API_OUTPUT_COUNT) {
+			return;
+		}
+
+		$resource = DBselect(
 			'SELECT hm.hostmacroid,hg.groupid'.
-			' FROM hostmacro hm,hosts_groups hg'.
-			' WHERE '.dbConditionInt('hm.hostmacroid', array_keys($result)).
-				' AND hm.hostid=hg.hostid'
+			' FROM hostmacro hm'.
+			' JOIN hosts_groups hg ON hm.hostid=hg.hostid'.
+			' JOIN hstgrp hgg ON hg.groupid=hgg.groupid'.
+				' AND '.dbConditionInt('hgg.type', [HOST_GROUP_TYPE_HOST_GROUP]).
+			' WHERE '.dbConditionId('hm.hostmacroid', array_keys($result))
 		);
-		$relationMap = new CRelationMap();
-		while ($relation = DBfetch($res)) {
-			$relationMap->addRelation($relation['hostmacroid'], $relation['groupid']);
+		$relation_map = new CRelationMap();
+
+		while ($relation = DBfetch($resource)) {
+			$relation_map->addRelation($relation['hostmacroid'], $relation['groupid']);
 		}
 
-		switch ($option) {
-			case 'selectGroups':
-				$output_tag = 'groups';
-				$entities = [API::HostGroup(), API::TemplateGroup()];
-				break;
-
-			case 'selectHostGroups':
-				$entities = [API::HostGroup()];
-				$output_tag = 'hostgroups';
-				break;
-
-			case 'selectTemplateGroups':
-				$entities = [API::TemplateGroup()];
-				$output_tag = 'templategroups';
-				break;
-		}
-
-		$groups = [];
-		foreach ($entities as $entity) {
-			$groups += $entity->get([
-				'output' => $options[$option],
-				'groupids' => $relationMap->getRelatedIds(),
+		$related_ids = $relation_map->getRelatedIds();
+		$groups = $related_ids
+			? API::HostGroup()->get([
+				'output' => $options['selectHostGroups'],
+				'groupids' => $related_ids,
 				'preservekeys' => true
-			]);
+			])
+			: [];
+
+		$result = $relation_map->mapMany($result, $groups, 'hostgroups');
+	}
+
+	private function addRelatedTemplateGroups(array $options, array &$result): void {
+		if ($options['selectTemplateGroups'] === null || $options['selectTemplateGroups'] === API_OUTPUT_COUNT) {
+			return;
 		}
 
-		$result = $relationMap->mapMany($result, $groups, $output_tag);
+		$resource = DBselect(
+			'SELECT hm.hostmacroid,hg.groupid'.
+			' FROM hostmacro hm'.
+			' JOIN hosts_groups hg ON hm.hostid=hg.hostid'.
+			' JOIN hstgrp hgg ON hg.groupid=hgg.groupid'.
+				' AND '.dbConditionInt('hgg.type', [HOST_GROUP_TYPE_TEMPLATE_GROUP]).
+			' WHERE '.dbConditionId('hm.hostmacroid', array_keys($result))
+		);
+		$relation_map = new CRelationMap();
+
+		while ($relation = DBfetch($resource)) {
+			$relation_map->addRelation($relation['hostmacroid'], $relation['groupid']);
+		}
+
+		$related_ids = $relation_map->getRelatedIds();
+		$groups = $related_ids
+			? API::TemplateGroup()->get([
+				'output' => $options['selectTemplateGroups'],
+				'groupids' => $related_ids,
+				'preservekeys' => true
+			])
+			: [];
+
+		$result = $relation_map->mapMany($result, $groups, 'templategroups');
+	}
+
+	private function addRelatedTemplates(array $options, array &$result): void {
+		if ($options['selectTemplates'] === null || $options['selectTemplates'] === API_OUTPUT_COUNT) {
+			return;
+		}
+
+		$relation_map = $this->createRelationMap($result, 'hostmacroid', 'hostid');
+		$templates = API::Template()->get([
+			'output' => $options['selectTemplates'],
+			'templateids' => $relation_map->getRelatedIds(),
+			'preservekeys' => true
+		]);
+
+		$result = $relation_map->mapMany($result, $templates, 'templates');
+	}
+
+	private function addRelatedHosts(array $options, array &$result): void {
+		if ($options['selectHosts'] === null || $options['selectHosts'] === API_OUTPUT_COUNT) {
+			return;
+		}
+
+		$relation_map = $this->createRelationMap($result, 'hostmacroid', 'hostid');
+		$hosts = API::Host()->get([
+			'output' => $options['selectHosts'],
+			'hostids' => $relation_map->getRelatedIds(),
+			'preservekeys' => true
+		]);
+
+		$result = $relation_map->mapMany($result, $hosts, 'hosts');
 	}
 
 	protected function unsetExtraFields(array $objects, array $fields, $output = []) {
