@@ -1043,7 +1043,9 @@ class CScreenProblem extends CScreenBase {
 				$header[] = $header_clock;
 			}
 
-			$table = (new CTableInfo())->setPageNavigation($paging);
+			$table = (new CTableInfo())
+				->setPageNavigation($paging)
+				->addClass($this->data['filter']['highlight_row'] == ZBX_HIGHLIGHT_ON ? 'has-highlighted-rows' : null);
 
 			// Create table.
 			if ($this->data['filter']['compact_view']) {
@@ -1151,10 +1153,13 @@ class CScreenProblem extends CScreenBase {
 
 			$footer = new CActionButtonList('action', 'eventids', [
 				'acknowledge.edit' => [
-					'name' => _('Mass update'),
-					'disabled' => !($allowed['add_comments'] || $allowed['change_severity'] || $allowed['acknowledge']
+					'content' => (new CSimpleButton(_('Mass update')))
+						->addClass(ZBX_STYLE_BTN_ALT)
+						->addClass('js-massupdate-problem')
+						->addClass('js-no-chkbxrange')
+						->setEnabled($allowed['add_comments'] || $allowed['change_severity'] || $allowed['acknowledge']
 							|| $allowed['close'] || $allowed['suppress_problems'] || $allowed['rank_change']
-					)
+						)
 				]
 			], 'problem');
 
@@ -1647,10 +1652,7 @@ class CScreenProblem extends CScreenBase {
 			$problem_update_link = ($data['allowed']['add_comments'] || $data['allowed']['change_severity']
 					|| $data['allowed']['acknowledge'] || $can_be_closed || $data['allowed']['suppress_problems']
 					|| $data['allowed']['rank_change'])
-				? (new CLink(_('Update'), $problem_update_url))
-					->addClass(ZBX_STYLE_LINK_ALT)
-					->setAttribute('data-eventids[]', $problem['eventid'])
-					->setAttribute('data-action', 'acknowledge.edit')
+				? (new CLink(_('Update'), $problem_update_url))->addClass(ZBX_STYLE_LINK_ALT)
 				: new CSpan(_('Update'));
 
 			$row->addItem([
@@ -1672,13 +1674,15 @@ class CScreenProblem extends CScreenBase {
 					: zbx_date2age($problem['clock']),
 				$problem_update_link,
 				makeEventActionsIcons($problem['eventid'], $data['actions'], $data['users'], $is_acknowledged),
-				$data['filter']['show_tags'] ? $data['tags'][$problem['eventid']] : null
+				$data['filter']['show_tags']
+					? (new CDiv($data['tags'][$problem['eventid']]))->addClass(ZBX_STYLE_TAGS_WRAPPER)
+					: null
 			]);
 
 			// Add table row.
-			$table->addRow($row, ($data['filter']['highlight_row'] && $value == TRIGGER_VALUE_TRUE)
-					? self::getSeverityFlhStyle($problem['severity'])
-					: null
+			$table->addRow($row, $data['filter']['highlight_row'] == ZBX_HIGHLIGHT_ON && $value == TRIGGER_VALUE_TRUE
+				? CSeverityHelper::getSeverityFlhStyle($problem['severity'])
+				: null
 			);
 
 			if ($problem['cause_eventid'] == 0 && $problem['symptoms']) {
@@ -1830,7 +1834,8 @@ class CScreenProblem extends CScreenBase {
 						: $last_value['value']
 				))
 					->addClass('hint-item')
-					->setAttribute('data-hintbox', '1');
+					->setAttribute('data-hintbox', '1')
+					->addClass(ZBX_STYLE_NO_INDENT);
 				$latest_values[] = ', ';
 			}
 			else {
@@ -1841,45 +1846,14 @@ class CScreenProblem extends CScreenBase {
 		}
 
 		if ($html) {
-			$hint_container = (new CDiv())
-				->addClass(ZBX_STYLE_HINTBOX_WRAP)
-				->addItem($hint_table);
-
 			array_pop($latest_values);
 			array_unshift($latest_values, (new CDiv())
 				->addClass('main-hint')
-				->setHint($hint_container)
+				->setHint($hint_table)
 			);
-
 			return $latest_values;
 		}
 
 		return implode(', ', $latest_values);
 	}
-
-	/**
-	 * Get trigger severity full line height css style name.
-	 *
-	 * @param int $severity  Trigger severity.
-	 *
-	 * @return string|null
-	 */
-	private static function getSeverityFlhStyle($severity) {
-		switch ($severity) {
-			case TRIGGER_SEVERITY_DISASTER:
-				return ZBX_STYLE_FLH_DISASTER_BG;
-			case TRIGGER_SEVERITY_HIGH:
-				return ZBX_STYLE_FLH_HIGH_BG;
-			case TRIGGER_SEVERITY_AVERAGE:
-				return ZBX_STYLE_FLH_AVERAGE_BG;
-			case TRIGGER_SEVERITY_WARNING:
-				return ZBX_STYLE_FLH_WARNING_BG;
-			case TRIGGER_SEVERITY_INFORMATION:
-				return ZBX_STYLE_FLH_INFO_BG;
-			case TRIGGER_SEVERITY_NOT_CLASSIFIED:
-				return ZBX_STYLE_FLH_NA_BG;
-			default:
-				return null;
-		}
-	}
 }
