@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -48,8 +43,7 @@ function DBconnect(&$error) {
 
 	$db_types = [
 		ZBX_DB_MYSQL => MysqlDbBackend::class,
-		ZBX_DB_POSTGRESQL => PostgresqlDbBackend::class,
-		ZBX_DB_ORACLE => OracleDbBackend::class
+		ZBX_DB_POSTGRESQL => PostgresqlDbBackend::class
 	];
 
 	if (!array_key_exists($DB['TYPE'], $db_types)) {
@@ -81,7 +75,7 @@ function DBconnect(&$error) {
 	return true;
 }
 
-function DBclose() {
+function DBclose(): bool {
 	global $DB;
 
 	$result = false;
@@ -94,44 +88,30 @@ function DBclose() {
 			case ZBX_DB_POSTGRESQL:
 				$result = pg_close($DB['DB']);
 				break;
-			case ZBX_DB_ORACLE:
-				$result = oci_close($DB['DB']);
-				break;
 		}
 	}
 	unset($DB['DB']);
+
 	return $result;
 }
 
-function DBstart() {
+function DBstart(): bool {
 	global $DB;
 
-	$result = false;
-
 	if ($DB['TRANSACTIONS'] != 0) {
 		info('POSSIBLE ERROR: Used incorrect logic in database processing, started subtransaction!');
-		return $result;
+
+		return false;
 	}
 
 	$DB['TRANSACTIONS']++;
 	$DB['TRANSACTION_NO_FAILED_SQLS'] = true;
 
 	if (!isset($DB['DB']) || empty($DB['DB'])) {
-		return $result;
+		return false;
 	}
 
-	switch ($DB['TYPE']) {
-		case ZBX_DB_MYSQL:
-			$result = DBexecute('BEGIN');
-			break;
-		case ZBX_DB_POSTGRESQL:
-			$result = DBexecute('BEGIN');
-			break;
-		case ZBX_DB_ORACLE:
-			$result = true;
-			break;
-	}
-	return $result;
+	return DBexecute('BEGIN');
 }
 
 /**
@@ -169,42 +149,12 @@ function DBend($doCommit = true) {
 	return (!is_null($doCommit) && $DBresult) ? $doCommit : $DBresult;
 }
 
-function DBcommit() {
-	global $DB;
-
-	$result = false;
-
-	switch ($DB['TYPE']) {
-		case ZBX_DB_MYSQL:
-			$result = DBexecute('COMMIT');
-			break;
-		case ZBX_DB_POSTGRESQL:
-			$result = DBexecute('COMMIT');
-			break;
-		case ZBX_DB_ORACLE:
-			$result = oci_commit($DB['DB']);
-			break;
-	}
-	return $result;
+function DBcommit(): bool {
+	return DBexecute('COMMIT');
 }
 
-function DBrollback() {
-	global $DB;
-
-	$result = false;
-
-	switch ($DB['TYPE']) {
-		case ZBX_DB_MYSQL:
-			$result = DBexecute('ROLLBACK');
-			break;
-		case ZBX_DB_POSTGRESQL:
-			$result = DBexecute('ROLLBACK');
-			break;
-		case ZBX_DB_ORACLE:
-			$result = oci_rollback($DB['DB']);
-			break;
-	}
-	return $result;
+function DBrollback(): bool {
+	return DBexecute('ROLLBACK');
 }
 
 /**
@@ -214,23 +164,25 @@ function DBrollback() {
  * DBselect('select * from users')
  * DBselect('select * from users',50,200)
  *
- * @param string $query
- * @param int $limit    max number of record to return
- * @param int $offset   return starting from $offset record
+ * @param string   $query
+ * @param int|null $limit   Maximum number of records to return.
+ * @param int      $offset  Return starting from $offset record.
  *
  * @return resource|false
  */
-function DBselect($query, $limit = null, $offset = 0) {
+function DBselect(string $query, ?int $limit = null, int $offset = 0) {
 	global $DB;
 
 	if (!array_key_exists('DB', $DB) || $DB['DB'] === null) {
 		return false;
 	}
 
-	$query = DBaddLimit($query, $limit, $offset);
+	if ($limit !== null) {
+		$query .= ' LIMIT '.$limit;
 
-	if ($query === false) {
-		return false;
+		if ($offset != 0) {
+			$query .= ' OFFSET '.$offset;
+		}
 	}
 
 	$time_start = microtime(true);
@@ -245,31 +197,14 @@ function DBselect($query, $limit = null, $offset = 0) {
 				$result = mysqli_query($DB['DB'], $query);
 			}
 			catch (mysqli_sql_exception $e) {
-				error('Error in query ['.$query.'] ['.$e->getMessage().']', 'sql');
+				trigger_error('Error in query ['.$query.'] ['.$e->getMessage().']', E_USER_WARNING);
 			}
 
 			break;
 
 		case ZBX_DB_POSTGRESQL:
 			if (!$result = pg_query($DB['DB'], $query)) {
-				error('Error in query ['.$query.'] ['.pg_last_error($DB['DB']).']', 'sql');
-			}
-
-			break;
-
-		case ZBX_DB_ORACLE:
-			$result = oci_parse($DB['DB'], $query);
-
-			if ($result === false) {
-				$e = oci_error();
-				error('SQL error ['.$e['message'].'] in ['.$e['sqltext'].']', 'sql');
-
-				break;
-			}
-
-			if (!@oci_execute($result, ($DB['TRANSACTIONS'] ? OCI_DEFAULT : OCI_COMMIT_ON_SUCCESS))) {
-				$e = oci_error($result);
-				error('SQL error ['.$e['message'].'] in ['.$e['sqltext'].']', 'sql');
+				trigger_error('Error in query ['.$query.'] ['.pg_last_error($DB['DB']).']', E_USER_WARNING);
 			}
 
 			break;
@@ -287,59 +222,6 @@ function DBselect($query, $limit = null, $offset = 0) {
 	return $result;
 }
 
-/**
- * Add the LIMIT clause to the given query.
- *
- * NOTE:
- * LIMIT and OFFSET records
- *
- * Example: select 6-15 row.
- *
- * MySQL:
- * SELECT a FROM tbl LIMIT 5,10
- * SELECT a FROM tbl LIMIT 10 OFFSET 5
- *
- * PostgreSQL:
- * SELECT a FROM tbl LIMIT 10 OFFSET 5
- *
- * Oracle:
- * SELECT a FROM tbl WHERE rownum < 15 // ONLY < 15
- * SELECT * FROM (SELECT * FROM tbl) WHERE rownum BETWEEN 6 AND 15
- *
- * @param $query
- * @param int $limit    max number of record to return
- * @param int $offset   return starting from $offset record
- *
- * @return bool|string
- */
-function DBaddLimit($query, $limit = 0, $offset = 0) {
-	global $DB;
-
-	if ((isset($limit) && ($limit < 0 || !zbx_ctype_digit($limit))) || $offset < 0 || !zbx_ctype_digit($offset)) {
-		$moreDetails = isset($limit) ? ' Limit ['.$limit.'] Offset ['.$offset.']' : ' Offset ['.$offset.']';
-		error('Incorrect parameters for limit and/or offset. Query ['.$query.']'.$moreDetails, 'sql');
-
-		return false;
-	}
-
-	// Process limit and offset
-	if (isset($limit)) {
-		switch ($DB['TYPE']) {
-			case ZBX_DB_MYSQL:
-			case ZBX_DB_POSTGRESQL:
-				$query .= ' LIMIT '.intval($limit);
-				$query .= $offset != 0 ? ' OFFSET '.intval($offset) : '';
-				break;
-			case ZBX_DB_ORACLE:
-				$till = $offset + $limit;
-				$query = 'SELECT * FROM ('.$query.') WHERE rownum BETWEEN '.intval($offset).' AND '.intval($till);
-				break;
-		}
-	}
-
-	return $query;
-}
-
 /**
  * @param $query
  *
@@ -364,35 +246,16 @@ function DBexecute($query): bool {
 				$result = mysqli_query($DB['DB'], $query);
 			}
 			catch (mysqli_sql_exception $e) {
-				error('Error in query ['.$query.'] ['.$e->getMessage().']', 'sql');
+				trigger_error('Error in query ['.$query.'] ['.$e->getMessage().']', E_USER_WARNING);
 			}
 
 			break;
 
 		case ZBX_DB_POSTGRESQL:
 			if (!$result = (bool) pg_query($DB['DB'], $query)) {
-				error('Error in query ['.$query.'] ['.pg_last_error($DB['DB']).']', 'sql');
+				error('Error in query ['.$query.'] ['.pg_last_error($DB['DB']).']', true);
 			}
 
-			break;
-
-		case ZBX_DB_ORACLE:
-			$result = oci_parse($DB['DB'], $query);
-
-			if ($result === false) {
-				$e = oci_error();
-				error('SQL error ['.$e['message'].'] in ['.$e['sqltext'].']', 'sql');
-
-				break;
-			}
-
-			if (!@oci_execute($result, ($DB['TRANSACTIONS'] ? OCI_DEFAULT : OCI_COMMIT_ON_SUCCESS))) {
-				$e = oci_error($result);
-				error('SQL error ['.$e['message'].'] in ['.$e['sqltext'].']', 'sql');
-			}
-
-			$result = true;
-
 			break;
 	}
 
@@ -450,28 +313,6 @@ function DBfetch($cursor, $convertNulls = true) {
 				pg_free_result($cursor);
 			}
 
-			break;
-
-		case ZBX_DB_ORACLE:
-			if ($row = oci_fetch_assoc($cursor)) {
-				$result = [];
-
-				foreach ($row as $key => $value) {
-					$field_type = strtolower(oci_field_type($cursor, $key));
-
-					// Since Oracle reports nulls for empty strings, convert those back to empty strings.
-					$value = (str_in_array($field_type, ['varchar', 'varchar2', 'blob', 'clob']) && is_null($value))
-						? ''
-						: $value;
-
-					if (is_object($value) && (strpos($field_type, 'lob') !== false)) {
-						$value = $value->load();
-					}
-
-					$result[strtolower($key)] = $value;
-				}
-			}
-
 			break;
 	}
 
@@ -491,7 +332,7 @@ function DBfetch($cursor, $convertNulls = true) {
 }
 
 function zbx_sql_mod($x, $y) {
-	return ' MOD('.$x.','.$y.')';
+	return 'MOD('.$x.','.$y.')';
 }
 
 function get_dbid($table, $field) {
@@ -553,7 +394,6 @@ function get_dbid($table, $field) {
 }
 
 function zbx_db_search($table, $options, &$sql_parts) {
-	global $DB;
 	list($table, $tableShort) = explode(' ', $table);
 
 	$tableSchema = DB::getSchema($table);
@@ -567,7 +407,8 @@ function zbx_db_search($table, $options, &$sql_parts) {
 
 	$search = [];
 	foreach ($options['search'] as $field => $patterns) {
-		if (!isset($tableSchema['fields'][$field]) || $patterns === null) {
+		if ($patterns === null || !array_key_exists($field, $tableSchema['fields'])
+				|| ($tableSchema['fields'][$field]['type'] & DB::SUPPORTED_SEARCH_TYPES) == 0) {
 			continue;
 		}
 
@@ -579,13 +420,6 @@ function zbx_db_search($table, $options, &$sql_parts) {
 			continue;
 		}
 
-		if ($tableSchema['fields'][$field]['type'] !== DB::FIELD_TYPE_CHAR
-				&& $tableSchema['fields'][$field]['type'] !== DB::FIELD_TYPE_NCLOB
-				&& $tableSchema['fields'][$field]['type'] !== DB::FIELD_TYPE_TEXT
-				&& $tableSchema['fields'][$field]['type'] !== DB::FIELD_TYPE_CUID) {
-			continue;
-		}
-
 		$fieldSearch = [];
 		foreach ($patterns as $pattern) {
 			// escaping parameter that is about to be used in LIKE statement
@@ -595,16 +429,9 @@ function zbx_db_search($table, $options, &$sql_parts) {
 				? $start.$pattern.'%'
 				: str_replace('*', '%', $pattern);
 
-			if ($DB['TYPE'] == ZBX_DB_ORACLE && $tableSchema['fields'][$field]['type'] === DB::FIELD_TYPE_NCLOB
-					&& strlen($pattern) > ORACLE_MAX_STRING_SIZE) {
-				$chunks = zbx_dbstr(DB::chunkMultibyteStr($pattern, ORACLE_MAX_STRING_SIZE));
-				$pattern = 'TO_NCLOB('.implode(') || TO_NCLOB(', $chunks).')';
-			}
-			else {
-				$pattern = zbx_dbstr($pattern);
-			}
+			$pattern = zbx_dbstr($pattern);
 
-			$fieldSearch[] = 'UPPER('.$tableShort.'.'.$field.')'.$exclude.' LIKE '.$pattern." ESCAPE '!'";
+			$fieldSearch[] = DB::uppercaseField($field, $table, $tableShort).$exclude.' LIKE '.$pattern." ESCAPE '!'";
 		}
 
 		$search[$field] = '('.implode($glue, $fieldSearch).')';
@@ -656,107 +483,50 @@ function check_db_fields($dbFields, &$args) {
  *
  * @param string $field_name
  * @param array  $values
- * @param bool   $not_in        Create inverse condition.
- * @param bool   $zero_to_null  Cast zero to null.
+ * @param bool   $not_in              Create inverse condition.
+ * @param bool   $zero_includes_null  ID fields may have default DB column value either null or 0. Consider them
+ *                                    equivalent for 0 in the given values.
  *
  * @return string
  */
-function dbConditionInt($field_name, array $values, $not_in = false, $zero_to_null = false) {
-	global $DB;
-
-	$MIN_NUM_BETWEEN = 4; // Minimum number of consecutive values for using "BETWEEN <id1> AND <idN>".
-	$MAX_NUM_IN = 950; // Maximum number of values for using "IN (<id1>,<id2>,...,<idN>)".
-
+function dbConditionInt($field_name, array $values, $not_in = false, $zero_includes_null = false) {
 	if (is_bool(reset($values))) {
 		return $not_in ? '1=1' : '1=0';
 	}
 
 	$values = array_flip($values);
 
-	$has_zero = false;
+	$condition = '';
 
-	if ($zero_to_null && array_key_exists(0, $values)) {
-		$has_zero = true;
-		unset($values[0]);
+	if ($zero_includes_null && array_key_exists(0, $values)) {
+		$condition .= $field_name.($not_in ? ' IS NOT NULL' : ' IS NULL');
 	}
 
 	$values = array_keys($values);
 	natsort($values);
 	$values = array_values($values);
 
-	$intervals = [];
-	$singles = [];
+	$singles = array_map(function($value) {
+		return dbQuoteInt($value);
+	}, $values);
 
-	if ($DB['TYPE'] == ZBX_DB_ORACLE) {
-		// For better performance, use "BETWEEN" constructs for sequential integer values, for Oracle database.
-
-		for ($i = 0, $size = count($values); $i < $size; $i++) {
-			if ($i + $MIN_NUM_BETWEEN < $size && bcsub($values[$i + $MIN_NUM_BETWEEN], $values[$i]) == $MIN_NUM_BETWEEN) {
-				$interval_first = $values[$i];
-
-				// Search for the last sequential integer value.
-				for ($i += $MIN_NUM_BETWEEN; $i < $size && bcsub($values[$i], $values[$i - 1]) == 1; $i++);
-				$i--;
-
-				$interval_last = $values[$i];
-
-				// Save the first and last values of the sequential interval.
-				$intervals[] = [dbQuoteInt($interval_first), dbQuoteInt($interval_last)];
-			}
-			else {
-				$singles[] = dbQuoteInt($values[$i]);
-			}
-		}
-	}
-	else {
-		// For better performance, use only "IN" constructs all other databases, except Oracle.
-
-		$singles = array_map(function($value) {
-			return dbQuoteInt($value);
-		}, $values);
-	}
-
-	$condition = '';
-
-	// Process intervals.
-
-	foreach ($intervals as $interval) {
-		if ($condition !== '') {
-			$condition .= $not_in ? ' AND ' : ' OR ';
-		}
-
-		$condition .= ($not_in ? 'NOT ' : '').$field_name.' BETWEEN '.$interval[0].' AND '.$interval[1];
-	}
-
-	// Process individual values.
-
-	$single_chunks = array_chunk($singles, $MAX_NUM_IN);
+	// Limit maximum number of values for using in "IN (<id1>,<id2>,...,<idN>)".
+	$single_chunks = array_chunk($singles, 950);
+	$multiple_conditions = false;
 
 	foreach ($single_chunks as $chunk) {
 		if ($condition !== '') {
 			$condition .= $not_in ? ' AND ' : ' OR ';
+			$multiple_conditions = true;
 		}
 
-		if (count($chunk) == 1) {
-			$condition .= $field_name.($not_in ? '!=' : '=').$chunk[0];
-		}
-		else {
-			$condition .= $field_name.($not_in ? ' NOT' : '').' IN ('.implode(',', $chunk).')';
-		}
+		$condition .= count($chunk) == 1
+			? $field_name.($not_in ? '!=' : '=').$chunk[0]
+			: $field_name.($not_in ? ' NOT' : '').' IN ('.implode(',', $chunk).')';
 	}
 
-	if ($has_zero) {
-		if ($condition !== '') {
-			$condition .= $not_in ? ' AND ' : ' OR ';
-		}
-
-		$condition .= $field_name.($not_in ? ' IS NOT NULL' : ' IS NULL');
-	}
-
-	if (!$not_in) {
-		if ((int) $has_zero + count($intervals) + count($single_chunks) > 1) {
-			$condition = '('.$condition.')';
-		}
+	if (!$not_in && $multiple_conditions) {
+		$condition = '('.$condition.')';
 	}
 
 	return $condition;
@@ -821,8 +591,7 @@ function dbQuoteInt($value) {
 }
 
 /**
- * Return SQL for COALESCE like select. For fields with type NCHAR, NVARCHAR or NTEXT in Oracle NVL should be used
- * instead of COALESCE because it will not check that all arguments have same type.
+ * Return SQL for COALESCE like select.
  *
  * @param string     $field_name       Field name to be used in returned query part.
  * @param int|string $default_value    Default value to be returned.
@@ -830,13 +599,11 @@ function dbQuoteInt($value) {
  * @return string
  */
 function dbConditionCoalesce($field_name, $default_value, $alias = '') {
-	global $DB;
-
 	if (is_string($default_value)) {
 		$default_value = ($default_value == '') ? '\'\'' : zbx_dbstr($default_value);
 	}
 
-	$query = (($DB['TYPE'] == ZBX_DB_ORACLE) ? 'NVL(' : 'COALESCE(').$field_name.','.$default_value.')';
+	$query = 'COALESCE('.$field_name.','.$default_value.')';
 
 	if ($alias) {
 		$query .= ' AS '.$alias;
@@ -914,7 +681,6 @@ function pg_connect_escape($string) {
 
 /**
  * Escape string for safe usage in SQL queries.
- * Works for mysql, oracle, postgresql.
  *
  * @param array|string $var
  *
@@ -937,15 +703,6 @@ function zbx_dbstr($var) {
 			}
 			return "'".mysqli_real_escape_string($DB['DB'], $var)."'";
 
-		case ZBX_DB_ORACLE:
-			if (is_array($var)) {
-				foreach ($var as $vnum => $value) {
-					$var[$vnum] = "'".preg_replace('/\'/', '\'\'', $value)."'";
-				}
-				return $var;
-			}
-			return "'".preg_replace('/\'/','\'\'',$var)."'";
-
 		case ZBX_DB_POSTGRESQL:
 			if (is_array($var)) {
 				foreach ($var as $vnum => $value) {
@@ -961,8 +718,7 @@ function zbx_dbstr($var) {
 }
 
 /**
- * Creates db dependent string with sql expression that casts passed value to bigint.
- * Works for mysql, oracle, postgresql.
+ * Creates DB dependent string with SQL expression that casts passed value to bigint.
  *
  * @param int $field
  *
@@ -982,9 +738,6 @@ function zbx_dbcast_2bigint($field) {
 		case ZBX_DB_MYSQL:
 			return 'CAST('.$field.' AS UNSIGNED)';
 
-		case ZBX_DB_ORACLE:
-			return 'CAST('.$field.' AS NUMBER(20))';
-
 		default:
 			return false;
 	}
