@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -46,7 +46,7 @@ class TabIndicators {
 	getForm() {
 		const ACTION = document.querySelector('#action-form');
 		const AUTHENTICATION = document.querySelector('#authentication-form');
-		const GRAPH = document.querySelector('#widget-dialogue-form');
+		const GRAPH = document.querySelector('form.dashboard-widget-svggraph');
 		const HOST = document.querySelector('#host-form');
 		const HOST_DISCOVERY = document.querySelector('#host-discovery-form');
 		const HOST_PROTOTYPE = document.querySelector('#host-prototype-form');
@@ -54,7 +54,7 @@ class TabIndicators {
 		const ITEM_PROTOTYPE = document.querySelector('#item-prototype-form');
 		const MAP = document.querySelector('#sysmap-form');
 		const MEDIA_TYPE = document.querySelector('#media-type-form');
-		const PIE_CHART = document.querySelector('#widget-dialogue-form');
+		const PIE_CHART = document.querySelector('form.dashboard-widget-piechart');
 		const PROXY = document.querySelector('#proxy-form');
 		const SERVICE = document.querySelector('#service-form');
 		const SLA = document.querySelector('#sla-form');
@@ -62,6 +62,7 @@ class TabIndicators {
 		const TRIGGER = document.querySelector('#trigger-form');
 		const TRIGGER_PROTOTYPE = document.querySelector('#trigger-prototype-form');
 		const USER = document.querySelector('#user-form');
+		const USERPROFILE_NOTIFICATION = document.querySelector('#userprofile-notification-form');
 		const USER_GROUP = document.querySelector('#user-group-form');
 		const WEB_SCENARIO = document.querySelector('#webscenario-form');
 
@@ -104,6 +105,8 @@ class TabIndicators {
 				return TRIGGER_PROTOTYPE;
 			case !!USER:
 				return USER;
+			case !!USERPROFILE_NOTIFICATION:
+				return USERPROFILE_NOTIFICATION;
 			case !!USER_GROUP:
 				return USER_GROUP;
 			case !!WEB_SCENARIO:
@@ -358,9 +361,7 @@ class HostMacrosTabIndicatorItem extends TabIndicatorItem {
 	}
 
 	getValue() {
-		let macros = [...document.forms['host-form'].querySelectorAll('#tbl_macros .form_row')];
-
-		return macros
+		return [...document.querySelectorAll('#tbl_macros .form_row')]
 			.filter((row) => {
 				const macro = row.querySelector('textarea[name$="[macro]"]');
 				const inherited_type = row.querySelector('input[name$="[inherited_type]"]');
@@ -402,15 +403,13 @@ class HostPrototypeMacrosTabIndicatorItem extends TabIndicatorItem {
 	}
 
 	getValue() {
-		let macros = [...document.forms['host-prototype-form'].querySelectorAll('#tbl_macros .form_row')];
-
-		return macros
+		return [...document.querySelectorAll('#tbl_macros .form_row')]
 			.filter((row) => {
 				const macro = row.querySelector('textarea[name$="[macro]"]');
 				const inherited_type = row.querySelector('input[name$="[inherited_type]"]');
-				const inherited = HostPrototypeMacrosTabIndicatorItem.ZBX_PROPERTY_INHERITED;
 
-				if (inherited_type !== null && parseInt(inherited_type.value, 10) == inherited) {
+				if (inherited_type !== null
+						&& parseInt(inherited_type.value, 10) == HostMacrosTabIndicatorItem.ZBX_PROPERTY_INHERITED) {
 					return false;
 				}
 
@@ -446,15 +445,13 @@ class TemplateMacrosTabIndicatorItem extends TabIndicatorItem {
 	}
 
 	getValue() {
-		let macros = [...document.forms['templates-form'].querySelectorAll('#tbl_macros .form_row')];
-
-		return macros
+		return [...document.querySelectorAll('#tbl_macros .form_row')]
 			.filter((row) => {
 				const macro = row.querySelector('textarea[name$="[macro]"]');
 				const inherited_type = row.querySelector('input[name$="[inherited_type]"]');
-				const inherited = TemplateMacrosTabIndicatorItem.ZBX_PROPERTY_INHERITED;
 
-				if (inherited_type !== null && parseInt(inherited_type.value, 10) == inherited) {
+				if (inherited_type !== null
+						&& parseInt(inherited_type.value, 10) == HostMacrosTabIndicatorItem.ZBX_PROPERTY_INHERITED) {
 					return false;
 				}
 
@@ -887,9 +884,10 @@ class FiltersTabIndicatorItem extends TabIndicatorItem {
 	}
 
 	getValue() {
-		return document
-			.querySelectorAll('#conditions tbody .form_row > td > input.macro:not(:placeholder-shown):not([readonly])')
-			.length;
+		const form_rows = document
+			.querySelectorAll('#conditions tbody .form_row > td > input.macro:not(:placeholder-shown)');
+
+		return [...form_rows].filter((row) => !row.readOnly || row.dataset.discovered).length;
 	}
 
 	initObserver() {
@@ -1359,6 +1357,7 @@ class SharingTabIndicatorItem extends TabIndicatorItem {
 class GraphAxesTabIndicatorItem extends TabIndicatorItem {
 
 	static SVG_GRAPH_AXIS_UNITS_AUTO = 0;
+	static SVG_GRAPH_AXIS_SCALE_LINEAR = 0;
 
 	constructor() {
 		super(TAB_INDICATOR_TYPE_MARK);
@@ -1377,6 +1376,12 @@ class GraphAxesTabIndicatorItem extends TabIndicatorItem {
 			}
 		}
 
+		for (const input of document.querySelectorAll('#lefty_scale, #righty_scale')) {
+			if (!input.disabled && input.value != GraphAxesTabIndicatorItem.SVG_GRAPH_AXIS_SCALE_LINEAR) {
+				return true;
+			}
+		}
+
 		for (const input of document.querySelectorAll('#lefty_units, #righty_units')) {
 			if (!input.disabled && input.value != GraphAxesTabIndicatorItem.SVG_GRAPH_AXIS_UNITS_AUTO) {
 				return true;
@@ -1474,6 +1479,7 @@ class GraphLegendTabIndicatorItem extends TabIndicatorItem {
 
 	static SVG_GRAPH_LEGEND_LINES_MIN = 1;
 	static SVG_GRAPH_LEGEND_COLUMNS_MAX = 4;
+	static SVG_GRAPH_LEGEND_LINES_MODE_FIXED = 0;
 
 	constructor() {
 		super(TAB_INDICATOR_TYPE_MARK);
@@ -1498,6 +1504,13 @@ class GraphLegendTabIndicatorItem extends TabIndicatorItem {
 			return true;
 		}
 
+		const legend_lines_mode = document.querySelector('[name="legend_lines_mode"]:checked');
+
+		if (legend_lines_mode !== null
+				&& legend_lines_mode.value != GraphLegendTabIndicatorItem.SVG_GRAPH_LEGEND_LINES_MODE_FIXED) {
+			return true;
+		}
+
 		const legend_lines = document.getElementById('legend_lines');
 
 		if (legend_lines !== null && legend_lines.value != GraphLegendTabIndicatorItem.SVG_GRAPH_LEGEND_LINES_MIN) {
@@ -1773,6 +1786,7 @@ class PieLegendTabIndicatorItem extends TabIndicatorItem {
 
 	static PIE_CHART_LEGEND_LINES_MIN = 1;
 	static PIE_CHART_LEGEND_COLUMNS_MAX = 4;
+	static PIE_CHART_LEGEND_LINES_MODE_FIXED = 0;
 
 	constructor() {
 		super(TAB_INDICATOR_TYPE_MARK);
@@ -1797,6 +1811,13 @@ class PieLegendTabIndicatorItem extends TabIndicatorItem {
 			return true;
 		}
 
+		const legend_lines_mode = document.querySelector('[name="legend_lines_mode"]:checked');
+
+		if (legend_lines_mode !== null
+				&& legend_lines_mode.value != PieLegendTabIndicatorItem.PIE_CHART_LEGEND_LINES_MODE_FIXED) {
+			return true;
+		}
+
 		const legend_lines = document.getElementById('legend_lines');
 
 		if (legend_lines !== null && legend_lines.value != PieLegendTabIndicatorItem.PIE_CHART_LEGEND_LINES_MIN) {
@@ -1806,7 +1827,7 @@ class PieLegendTabIndicatorItem extends TabIndicatorItem {
 		const legend_columns = document.getElementById('legend_columns');
 
 		if (legend_columns !== null
-			&& legend_columns.value != PieLegendTabIndicatorItem.PIE_CHART_LEGEND_COLUMNS_MAX) {
+				&& legend_columns.value != PieLegendTabIndicatorItem.PIE_CHART_LEGEND_COLUMNS_MAX) {
 			return true;
 		}
 
