@@ -1,3 +1,18 @@
+/*
+** Copyright (C) 2001-2025 Zabbix SIA
+**
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
+**
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
+**
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
+**/
+
+
 class ZSelect extends HTMLElement {
 
 	constructor() {
@@ -6,6 +21,7 @@ class ZSelect extends HTMLElement {
 		this._options_map = new Map();
 
 		this._option_template = "#{label}";
+		this._selected_option_template = "#{label}";
 
 		this._highlighted_index = -1;
 		this._preselected_index = -1;
@@ -18,7 +34,6 @@ class ZSelect extends HTMLElement {
 		this._list = document.createElement('ul');
 
 		this._events = {};
-		this.onchange = () => {};
 
 		this._is_connected = false;
 	}
@@ -38,7 +53,7 @@ class ZSelect extends HTMLElement {
 	 * @return {array}
 	 */
 	static get observedAttributes() {
-		return ['name', 'value', 'disabled', 'readonly', 'width', 'onchange'];
+		return ['name', 'value', 'disabled', 'readonly', 'width'];
 	}
 
 	attributeChangedCallback(name, old_value, new_value) {
@@ -56,7 +71,8 @@ class ZSelect extends HTMLElement {
 
 					if (option && !option.disabled) {
 						this._input.value = option.value;
-						this.dispatchEvent(new Event('change'));
+						this.dispatchEvent(new Event('change', {bubbles: true}));
+						this.dispatchEvent(new Event('input', {bubbles: true}));
 					}
 					else {
 						this._input.value = null;
@@ -75,18 +91,22 @@ class ZSelect extends HTMLElement {
 				break;
 
 			case 'width':
-				this._button.style.width = `${new_value}px`;
-				this._list.style.width = `${new_value}px`;
-				break;
-
-			case 'onchange':
-				this.onchange = new Function('e', new_value);
+				if (new_value === 'auto') {
+					this.style.width = '100%';
+				}
+				else if (new_value !== null) {
+					this.style.width = `${new_value}px`;
+				}
+				else {
+					this.style.width = '';
+				}
 				break;
 		}
 	}
 
 	init() {
 		this._button.type = 'button';
+		this._button.classList.add('focusable');
 		this.appendChild(this._button);
 
 		this._input.type = 'hidden';
@@ -105,6 +125,11 @@ class ZSelect extends HTMLElement {
 			this.removeAttribute('option-template');
 		}
 
+		if (this.hasAttribute('selected-option-template')) {
+			this._selected_option_template = this.getAttribute('selected-option-template');
+			this.removeAttribute('selected-option-template');
+		}
+
 		if (this.hasAttribute('data-options')) {
 			const options = JSON.parse(this.getAttribute('data-options'));
 
@@ -155,6 +180,7 @@ class ZSelect extends HTMLElement {
 
 		li._index = this._options_map.size;
 		li.setAttribute('value', value);
+		li.setAttribute('title', label.trim().replace(/<[^>]*>/g, ''));
 		li.innerHTML = new Template(template || this._option_template).evaluate(
 			Object.assign({label: label.trim()}, extra || {})
 		);
@@ -177,6 +203,13 @@ class ZSelect extends HTMLElement {
 					option.is_disabled = is_disabled;
 					li.toggleAttribute('disabled', is_disabled);
 				}
+			},
+			hidden: {
+				set: (is_hidden) => {
+					option.disabled = is_hidden;
+					is_hidden ? li.setAttribute('disabled', 'disabled') : li.removeAttribute('disabled');
+					li.style.display = is_hidden ? 'none' : '';
+				}
 			}
 		}));
 
@@ -214,6 +247,10 @@ class ZSelect extends HTMLElement {
 		this.setAttribute('name', name);
 	}
 
+	get options() {
+		return this.getOptions();
+	}
+
 	get value() {
 		return !this._is_connected
 			? this.getAttribute('value')
@@ -249,7 +286,13 @@ class ZSelect extends HTMLElement {
 	}
 
 	_expand() {
-		const {height: button_height, y: button_y, left: button_left} = this._button.getBoundingClientRect();
+		const {
+			width: button_width,
+			height: button_height,
+			y: button_y,
+			left: button_left,
+			right: button_right
+		} = this._button.getBoundingClientRect();
 		const {height: document_height} = document.body.getBoundingClientRect();
 
 		if (button_y + button_height < 0 && document_height - button_y < 0) {
@@ -267,6 +310,7 @@ class ZSelect extends HTMLElement {
 		const space_above = button_y;
 
 		this._list.style.left = `${button_left}px`;
+		this._list.style.minWidth = `${button_width}px`;
 		this._list.style.maxHeight = '';
 
 		if (space_below - list_height > offset_bottom || space_below > space_above) {
@@ -288,10 +332,18 @@ class ZSelect extends HTMLElement {
 			}
 		}
 
+		const screen_width = window.innerWidth;
+		const list_right_edge = this._list.offsetLeft + this._list.offsetWidth;
+
+		if (list_right_edge > screen_width) {
+			this._list.style.left = '';
+			this._list.style.right = `${screen_width - button_right}px`;
+		}
+
 		this._highlight(this._preselected_index);
 
 		document.addEventListener('wheel', this._events.document_wheel);
-		document.querySelector('.wrapper').addEventListener('scroll', this._events.document_wheel);
+		setTimeout(() => document.querySelector('.wrapper').addEventListener('scroll', this._events.document_wheel));
 	}
 
 	_collapse() {
@@ -325,7 +377,9 @@ class ZSelect extends HTMLElement {
 		const option = this.getOptionByIndex(index);
 
 		if (option) {
-			this._button.innerText = option.label;
+			this._button.innerHTML = new Template(this._selected_option_template).evaluate(
+				Object.assign({label: option.label.trim()}, option.extra || {})
+			);
 			this._input.disabled = this.hasAttribute('disabled');
 		}
 		else {
@@ -334,6 +388,7 @@ class ZSelect extends HTMLElement {
 		}
 
 		this._preselected_index = index;
+		this._highlighted_index = index;
 	}
 
 	_change(index) {
@@ -433,7 +488,7 @@ class ZSelect extends HTMLElement {
 
 		container.remove();
 
-		return width;
+		return Math.min(width, 453);
 	}
 
 	_isVisible() {
@@ -452,6 +507,10 @@ class ZSelect extends HTMLElement {
 	registerEvents() {
 		this._events = {
 			button_mousedown: (e) => {
+				if (this._button.readOnly) {
+					return;
+				}
+
 				// Safari fix - event needs to be prevented, else blur event is fired on this button.
 				e.preventDefault();
 				// Safari fix - a click button on label would not focus button element.
@@ -469,6 +528,10 @@ class ZSelect extends HTMLElement {
 			},
 
 			button_keydown: (e) => {
+				if (this._button.readOnly) {
+					return;
+				}
+
 				!this._isVisible() && this.scrollIntoView({block: 'nearest'})
 
 				if (e.which !== KEY_SPACE && !e.metaKey && !e.ctrlKey && e.key.length === 1) {
@@ -568,7 +631,15 @@ class ZSelect extends HTMLElement {
 				document.activeElement !== this._button && this._button.focus();
 			},
 
+			focus: (e) => {
+				this._button.focus();
+			},
+
 			button_blur: () => {
+				if (this._button.readOnly) {
+					return;
+				}
+
 				this._change(this._preselected_index);
 				this._collapse();
 			},
@@ -628,6 +699,8 @@ class ZSelect extends HTMLElement {
 		this._list.addEventListener('mouseup', this._events.list_mouseup);
 		this._list.addEventListener('mousemove', this._events.list_mousemove);
 
+		this.addEventListener('focus', this._events.focus);
+
 		window.addEventListener('resize', this._events.window_resize);
 	}
 
@@ -643,6 +716,8 @@ class ZSelect extends HTMLElement {
 		this._list.removeEventListener('mouseup', this._events.list_mouseup);
 		this._list.removeEventListener('mousemove', this._events.list_mousemove);
 
+		this.removeEventListener('focus', this._events.focus);
+
 		window.removeEventListener('resize', this._events.window_resize);
 	}
 }
