@@ -27,7 +27,7 @@ class CFieldTextBox extends CField {
 	}
 
 	getValue() {
-		if (this._field.disabled) {
+		if (this.isDisabled()) {
 			return null;
 		}
 
@@ -44,6 +44,10 @@ class CFieldTextBox extends CField {
 		return value.trim();
 	}
 
+	isDisabled() {
+		return this._field.disabled;
+	}
+
 	focusErrorField() {
 		super.focusErrorField();
 		this._field.focus();
